//-------------------------------------------------------------------------------------
// <copyright file="ImportBindings.cs" company="Microsoft">
//      Copyright (c) 2003 Microsoft Corporation. All rights reserved.
//      THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
//      EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
//      WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
// </copyright>
// <summary>
//      Imports the bindings of an assembly into BizTalk from an XML file.
// </summary>  
//-------------------------------------------------------------------------------------

namespace Microsoft.Sdc.Tasks.BizTalk2004.Assembly
{
    #region Using directives

    using System;
    using System.Collections.Generic;
    using System.Text;

    using Microsoft.Build.Framework;
    using Microsoft.Sdc.Tasks.BizTalk2004.Configuration;
    using Microsoft.Sdc.Tasks;

    #endregion

    #region Class Comments
    /// <summary>
    /// Imports the bindings of an assembly into BizTalk from an XML file.
    /// </summary>
    /// <remarks>
    /// <code>
    /// <![CDATA[<BizTalk2004.Assembly.ImportBindings BindingPath="bindingPath" AssemblyPath="assemblyPath" InstallInGac="installInGac" LogPath="logPath" Server="server" Database="database" />]]>
    /// </code>
    /// <para>where:</para>
    /// <para><i>bindingPath (Required)</i></para>
    /// <para>
    /// The path to the XML file containing the bindings.
    /// </para>
    /// <para><i>assemblyPath (Required)</i></para>
    /// <para>
    /// The path to the assembly to have its bindings imported.
    /// </para>
    /// <para><i>installInGac</i></para>
    /// <para>
    /// <b>true</b> to install the assembly in the GAC, or <b>false</b> otherwise. The default is <b>false</b>.
    /// </para>
    /// <para><i>logPath</i></para>
    /// <para>
    /// The path to the log file for the binding import. The default is no log file.
    /// </para>
    /// <para><i>server</i></para>
    /// <para>
    /// The logical name of the server hosting the BizTalk management database. If neither this nor the
    /// database name are specified then the installation will default to the local installation.
    /// </para>
    /// <para><i>database</i></para>
    /// <para>
    /// The name of the BizTalk management database. If neither this nor the server name are specified 
    /// then the installation will default to the local installation.
    /// </para>
    /// <para><i>username</i></para>
    /// <para>
    /// The username to use to connect to the database if using SQL Server security. The default is to
    /// connect using Windows Integrated security.
    /// </para>
    /// <para><i>password</i></para>
    /// <para>
    /// The password to use to connect to the database if using SQL Server security. The default is to
    /// connect using Windows Integrated security.
    /// </para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <BizTalk2004.Assembly.ImportBindings 
    ///             AssemblyPath="C:\Build\Output\MyAssembly.dll" 
    ///             BindingPath="C:\Build\Bindings\MyAssemblyBindings.xml"
    ///             LogPath="C:\Build\Logs\MyAssemblyImportBindings.log" 
    ///             Server="." 
    ///             Database="BizTalkMgmtDb" 
    ///              
    ///              />
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>
    #endregion
    public class ImportBindings : BizTalk2004TaskBase
    {
        #region Member Variables

        private string assemblyPath;
        private string bindingPath;
        private bool installInGac;
        private string logPath;

        #endregion

        #region Constructors

        /// <summary>
        /// Creates a new instance of the import bindings task.
        /// </summary>
        public ImportBindings()
        {
        }

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the path to the assembly to undeploy.
        /// </summary>
        /// <value>
        /// The path to the assembly to undeploy.
        /// </value>
        [Required]
        public string AssemblyPath
        {
            get
            {
                return this.assemblyPath;
            }
            set
            {
                this.assemblyPath = value;
            }
        }

        /// <summary>
        /// Gets or sets the path to the assembly binding XML file.
        /// </summary>
        /// <value>
        /// The path to the assembly binding XML file.
        /// </value>
        [Required]
        public string BindingPath
        {
            get
            {
                return this.bindingPath;
            }
            set
            {
                this.bindingPath = value;
            }
        }

        /// <summary>
        /// Gets or sets whether the assembly will be installed into the Global Assembly Cache (GAC).
        /// </summary>
        /// <value>
        /// <b>true</b> to install the assembly into the GAC, or <b>false</b> otherwise. The default is
        /// <b>false</b>.
        /// </value>
        public bool InstallInGac
        {
            get
            {
                return this.installInGac;
            }
            set
            {
                this.installInGac = value;
            }
        }

        /// <summary>
        /// Gets or sets the path to the log file for the uninstallation.
        /// </summary>
        /// <value>
        /// The path to the log file for the uninstallation.
        /// </value>
        public string LogPath
        {
            get
            {
                return this.logPath;
            }
            set
            {
                this.logPath = value;
            }
        }

        #endregion

        #region Methods

        /// <summary>
        /// Executes the task.
        /// </summary>
        protected override void InternalExecute()
        {
            BizTalkInstallation installation = this.GetInstallation();
            BizTalkAssembly.ImportBindings(installation, this.bindingPath, this.logPath);
        }

        #endregion
    }
}

