//-----------------------------------------------------------------------
// <copyright file="GetIPByHostName.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Martin peck</author>
// <email>mpeck</email>
// <date>2006-02-21</date>
// <summary>Returns the IP addresses for a given hostname</summary>
//-----------------------------------------------------------------------
namespace Microsoft.Sdc.Tasks.Network
{
    using System;
    using System.Collections.Generic;
    using System.Text;
    using System.Globalization;
    using System.Net;
    using Microsoft.Build.Framework;

    /// <summary>
    /// Returns the IP addresses of a given hostname
    /// </summary>
    /// <remarks>
    /// <code><![CDATA[ 
    /// <Network.CheckHostNameExists HostName="www.microsoft.com">
    ///        <Output TaskParameter="HostExists" PropertyName="existsFlag" />    
    /// </Network.CheckHostNameExists>
    /// ]]></code>
    /// 
    /// <para>where:</para>
    /// <para><i>HostName (Required)</i></para>
    /// <para>hostname to resolve
    /// </para>
    /// <para>HostExists</para>
    /// <para> boolean. True if hostname is resolvable to an IP address
    /// </para>   
    /// </remarks>
    /// <example>
    /// <code><![CDATA[ 
    /// <Network.CheckHostNameExists HostName="www.microsoft.com">
    ///        <Output TaskParameter="HostExists" PropertyName="existsFlag" />    
    /// </Network.CheckHostNameExists>
    /// ]]></code>    
    /// </example>
    public class CheckHostNameExists : TaskBase
    {
        /// <summary>
        /// host name
        /// </summary>
        private string hostName;

        /// <summary>
        /// hostExists
        /// </summary>
        private bool hostExists;

        /// <summary>
        /// Treat Errors As Warnings
        /// </summary>
        private bool treatErrorsAsWarnings;

        /// <summary>
        /// Initializes a new instance of the GetIPByName class.
        /// </summary>
        public CheckHostNameExists()
        {
        }

        /// <summary>
        /// The host name to resolve
        /// </summary>
        [Required]
        public string HostName
        {
            get { return this.hostName; }
            set { this.hostName = value; }
        }

        /// <summary>
        /// Treat Errors As Warnings
        /// </summary>
        public bool TreatErrorsAsWarnings
        {
            get { return this.treatErrorsAsWarnings; }
            set { this.treatErrorsAsWarnings = value; }
        }

        /// <summary>
        /// HostExists
        /// </summary>
        [Output]
        public bool HostExists
        {
            get { return this.hostExists; }
        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            try
            {
                IPHostEntry hostEntry = Dns.GetHostEntry(this.hostName);
                this.hostExists = true;
            }
            catch (System.Net.Sockets.SocketException ex)
            {
                if (ex.SocketErrorCode == System.Net.Sockets.SocketError.HostNotFound)
                {
                    this.hostExists = false;
                }
                else
                {
                    LogError(ex);
                }
            }
        }

        private void LogError(Exception e)
        {
            if (this.treatErrorsAsWarnings)
            {
                base.Log.LogWarningWithCodeFromResources("Network.CheckHostNameExists.Failed", this.hostName, e.Message);
            }
            else
            {
                base.Log.LogErrorWithCodeFromResources("Network.CheckHostNameExists.Failed", this.hostName, e.Message);
            }
        }
    }
}
