//-----------------------------------------------------------------------
// <copyright file="LabelSync.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Giles Knap [Updated 03 Apr 08: Mike Fourie - Code Cleanup, TFS 2008 Support, Workingdirectory Support etc.]</author>
// <date>2006-03-23</date>
// <summary>Returns the Changes In The Team Source Interval Specified</summary>
//-----------------------------------------------------------------------
namespace Microsoft.Sdc.Tasks.SourceTfs
{
    using System;
    using System.IO;
    using System.Globalization;
    using Microsoft.Sdc.Tasks.Configuration;
    using Microsoft.Build.Framework;

    /// <summary>
    /// Sync the label specified with the current Workspace
    /// </summary>
    /// <remarks>
    /// <code><![CDATA[<SourceTfs.LabelSync Label="label" TfsVersion="tfsVersion" WorkingDirectory="workingDirectory"/>]]></code>
    /// <para>where:</para>
    /// <para><i>label (Required)</i></para>
    /// <para>The label to sync to.</para>
    /// <para><i>tfsVersion</i></para>
    /// <para>Version of TFS to use. Supports 2005 and 2008. Default is 2005</para>
    /// <para><i>workingDirectory</i></para>
    /// <para>Specifies the WorkingDirectory to run TF.exe in. This determines the workspace</para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <SourceTfs.LabelSync
    ///             Label="ConfigMainLatest" />
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>
    public class LabelSync : TaskBase
    {
        private string label;
        private string path = "$/"; // default to the whole depot
        private string tfsVersion = "2005";
        private string workingDirectory;
        private string outputFolder;

        /// <summary>
        /// Gets or sets the working directory.
        /// </summary>
        /// <value>The working directory.</value>
        public string WorkingDirectory
        {
            get { return this.workingDirectory; }
            set { this.workingDirectory = value; }
        }

        /// <summary>
        /// Gets or sets the path to the folder to write the sync command output to. If not specified it won't be written to a file.
        /// </summary>
        public string OutputFolder
        {
            get { return this.outputFolder; }
            set { this.outputFolder = value; }
        }

        /// <summary>
        /// Gets or sets the TFS version.
        /// </summary>
        /// <value>The TFS version.</value>
        public string TfsVersion
        {
            get { return this.tfsVersion; }
            set { this.tfsVersion = value; }
        }

        /// <summary>
        /// Gets or sets the path to the file or folder.
        /// </summary>
        /// <value>The full path to the file or folder.</value>
        public string Path
        {
            get { return this.path; }
            set { this.path = value; }
        }

        /// <summary>
        /// Gets or sets the  label to sync to
        /// </summary>
        /// <value>The label to sync to</value>
        [Required]
        public string Label
        {
            get { return this.label; }
            set { this.label = value; }
        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            string outputFilePath = null;
            if (string.IsNullOrEmpty(this.OutputFolder) == false)
            {
                outputFilePath = System.IO.Path.Combine(this.OutputFolder, "SourceTfsLabelSync.txt");

                if (!Directory.Exists(this.OutputFolder))
                {
                    Directory.CreateDirectory(this.outputFolder);
                }

                if (System.IO.File.Exists(outputFilePath))
                {
                    System.IO.File.Delete(outputFilePath);
                }
            } 
            
            Helper h = new Helper(this.TfsVersion);
            string arguments = String.Format(CultureInfo.InvariantCulture, "Label \"{0}\" \"{1}\" /recursive /noprompt /child:Replace", this.label, this.path);
            ShellExecute shellExecute = new ShellExecute(h.TfExePath, arguments);
            if (string.IsNullOrEmpty(this.WorkingDirectory) == false)
            {
                shellExecute.WorkingDirectory = this.WorkingDirectory;
            }

            Log.LogMessage(MessageImportance.Normal, string.Format("Executing {0} with {1}", shellExecute.Filename, arguments));

            int returnValue = shellExecute.Execute();

            // stdOut contains a list of files synced
            if (string.IsNullOrEmpty(this.OutputFolder) == false)
            {
                StreamWriter sw = System.IO.File.CreateText(outputFilePath);
                sw.Write(shellExecute.StandardOutput);
                sw.Close();
            }

            Log.LogMessage(MessageImportance.Low, shellExecute.StandardOutput);

            switch (returnValue)
            {
                case 1:
                    Log.LogWarning("Exit Code 1. Partial success: " + shellExecute.StandardError.Trim());
                    break;
                case 2:
                    Log.LogError("Exit Code 2. Unrecognized command: " + shellExecute.StandardError.Trim());
                    break;
                case 100:
                    Log.LogError("Exit Code 100. Nothing Succeeded: " + shellExecute.StandardError.Trim());
                    break;
            }
        }
    }
}