//-----------------------------------------------------------------------
// <copyright file="GetDatabaseVersion.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Giles Knap [2008-02-27: Mike Fourie - updated with codeplex feedback and code cleanup.</author>
// <email>gilesk</email>
// <date>2006-05-02</date>
// <summary>Gets the database version.</summary>
//-----------------------------------------------------------------------
namespace Microsoft.Sdc.Tasks.Sql
{
    using System;
    using System.IO;
    using System.Data;
    using System.Data.SqlClient;
    using SdcSql = Microsoft.Sdc.Tasks.Configuration.SqlServer;
    using Microsoft.Build.Framework;

    /// <summary>
    /// Gets an extended property 'DatabaseVersionNumber' on the database specified in the connection string. This can be used in
    /// conjunction with 'SetDatabaseVersion' to control db initialization/upgrade scripts in installers etc.
    /// 
    /// This task deliberately returns
    ///     0.0 if there is no existing database at all
    ///     1.0 if the db exists but does not yet have the 'DatabaseVersionNumber' extended property
    /// 
    ///
    /// </summary>
    /// <remarks>
    /// <code><![CDATA[<Sql.Execute DatabaseVersion="versionNumber" ServerName="serverName" DatabaseName="database" ConnectionString="oleDBConnectionString" CommandTimeout="timeout" />]]></code>
    /// <para>where:</para>
    /// <para><i>serverName</i></para>
    /// <para>The server hosting the database (ignored if a connection string is supplied)</para>
    /// <para><i>databaseName</i></para>
    /// <para>The database (ignored if a connection string is supplied)</para>
    /// <para>If you do not specify a server and database name you must specify a 
    /// <see cref="ConnectionString">connection string</see>.</para>
    /// <para><i>connectionString</i></para>
    /// <para>An OLE/DB connection string detailing the connection parameters required.</para>
    /// <para>If you do not specify a connection string you must specify a
    /// <see cref="ServerName">serverName</see> and <see cref="DatabaseName">databaseName</see></para>
    /// <para><i>timeout</i></para>
    /// <para>Maximum time allowed to perform the task (seconds)</para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <Sql.GetDatabaseVersion
    ///             ServerName="(local)"
    ///             DatabaseName="testDB"
    ///             CommandTimeout="60">
    ///             <Output TaskParameter="DatabaseVersion" PropertyName="DatabaseVersion"/>
    ///         </Sql.GetDatabaseVersion>
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>
    [Obsolete("Please review the functionality provided by the new Sql.ExtendedProperty class")]
    public class GetDatabaseVersion : TaskBase
    {
        private string connectionString = String.Empty;
        private string databaseVersion = String.Empty;
        private string serverName = String.Empty;
        private string databaseName = String.Empty;
        private int commandTimeout = 30;

        /// <summary>
        /// Set this property to define a database connection string. This
        /// property can be used instead of the combination of <see cref="Microsoft.Sdc.Tasks.Sql.GetDatabaseVersion.ServerName"/> and
        /// <see cref="Microsoft.Sdc.Tasks.Sql.GetDatabaseVersion.DatabaseName"/>.
        /// </summary>
        public string ConnectionString
        {
            get
            {
                return (this.connectionString ?? String.Empty);
            }

            set
            {
                this.connectionString = value;
                return;
            }
        }

        /// <summary>
        /// The version number to set 
        /// </summary>
        [Output]
        public string DatabaseVersion
        {
            get { return this.databaseVersion; }
            set { this.databaseVersion = value; }
        }

        /// <summary>
        /// The SQL Server name.
        /// </summary>
        public string ServerName
        {
            get
            {
                return (this.serverName ?? String.Empty);
            }

            set
            {
                this.serverName = value;
                return;
            }
        }

        /// <summary>
        /// Name of the database that the script will be run against.
        /// </summary>
        public string DatabaseName
        {
            get
            {
                return (this.databaseName ?? String.Empty);
            }

            set
            {
                this.databaseName = value;
                return;
            }
        }

        /// <summary>
        /// Gets or sets the wait time in seconds before terminating the 
        /// attempt to execute a command and generating an error.
        /// </summary>
        /// <remarks>
        /// Has a default value of 30 seconds.
        /// </remarks>
        public int CommandTimeout
        {
            get
            {
                return this.commandTimeout;
            }

            set
            {
                this.commandTimeout = value;
                return;
            }
        }

        /// <summary>
        /// Performs the action of this task which is to execute T-SQL scripts.
        /// </summary>
        protected override void InternalExecute()
        {
            string commandText;

            #region Execute code

            System.Reflection.Assembly assem = this.GetType().Assembly;
            using (Stream stream = assem.GetManifestResourceStream("Microsoft.Sdc.Tasks.Sql.GetDatabaseVersion.sql"))
            {
                using (StreamReader reader = new StreamReader(stream))
                {
                    commandText = reader.ReadToEnd();
                }
            }

            if (String.IsNullOrEmpty(this.ConnectionString))
            {
                this.ConnectionString = string.Format(
                    "Data Source={0};Integrated Security=SSPI;Initial Catalog={1}",
                    this.serverName,
                    this.DatabaseName);
            }

            using (SqlConnection sqlConnection = new SqlConnection(this.ConnectionString))
            {
                SqlCommand command = new SqlCommand(commandText, sqlConnection);
                command.CommandTimeout = this.CommandTimeout;
                bool connectedOK = true;

                SqlParameter databVersion = new SqlParameter("@DatabaseVersion", SqlDbType.NVarChar, 2000);
                databVersion.Direction = ParameterDirection.Output;

                command.Parameters.Add(databVersion);

                try
                {
                    sqlConnection.Open();
                }
                catch (SqlException ex)
                {
                    if (ex.Number == 4060)
                    {
                        // Database does not exist, we use version 0.0 to represent this
                        this.databaseVersion = "0.0";
                        connectedOK = false;
                    }
                    else
                    {
                        throw;
                    }
                }

                if (connectedOK)
                {
                    command.ExecuteNonQuery();
                    sqlConnection.Close();

                    this.databaseVersion = databVersion.Value.ToString();
                }
            }

            #endregion
        }
    }
}