//-----------------------------------------------------------------------
// <copyright file="Nunit.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// Nunit is 
//    Copyright  2002 James W. Newkirk, Michael C. Two, Alexei A. Vorontsov
//    Copyright  2000-2002 Philip A. Craig
//    
//    http://www.nunit.org/license.html
//   
//    For additional information and to obtain a copy of Nunit please go to
//    http://www.nunit.org
//    or for version 2.1
//    http://www.sourceforge.net
// </copyright>
// <author>James Hartill</author>
// <email></email>
// <date>2004-03-23</date>
// <summary>Executes all testcases in an assembly using Nunit.</summary>
//-----------------------------------------------------------------------

namespace Microsoft.Sdc.Tasks.Tools
{
    using System;
    using System.Xml;
    using System.Globalization;
    using System.IO;
    using System.Text;
    using System.Collections;
    using System.Reflection;
    using Microsoft.Build.Framework;
    using Microsoft.Build.Tasks;
    using Microsoft.Build.Utilities;

    /// <summary>
    /// <para>Runs Nunit, creates a summary file and returns test statistics.</para>
    /// <para>Testcases must be written according to the Nunit documentation.</para>
    /// <para>For more information and a copy of Nunit please visit
    /// <seealso href="http://www.nunit.org">http://www.nunit.org</seealso></para>     
    /// <para>Originally written for Nunit Version 2.0</para>
    /// <para>Updated to support Nunit version 2.1</para>
    /// </summary>
    /// <remarks>
    /// <code><![CDATA[
    /// <Tools.Nunit AssemblyName="assemblyname" 
    ///                    Version="version"
    ///                 Fixture="fixture"
    ///                 IncludeCategory="includecategory"
    ///                 ExcludeCategory="excludecategory"
    ///                 OutputXmlFileName="outputxmlfilename"
    ///                 OutputXmlTransformFile="outputxmltransformfile">
    ///     <Output TaskParameter="Category" ItemName="itemName" />
    ///     <Output TaskParameter="Pass" ItemName="itemName" />
    ///     <Output TaskParameter="Fail" ItemName="itemName" />
    ///     <Output TaskParameter="TimeTaken" ItemName="itemName" />
    /// </Tools.Nunit>
    /// ]]></code>
    /// <para>where:</para>
    /// <para><i>version</i></para>
    /// <para>Version of Nunit to run</para>
    /// <para><i>fixture</i></para>
    /// <para>Fully qualified fixture name if required</para>
    /// <para><i>includecategory</i></para>
    /// <para>List of test Categories to run ';' separated</para>
    /// <para><i>excludecategory</i></para>
    /// <para>List of test Categories to ignore ';' separated</para>
    /// <para><i>outputxmlfilename</i></para>
    /// <para>File name to write the results to</para>
    /// <para><i>outputxmltransformfile</i></para>
    /// <para>File name of transform file if required</para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test">
    ///         <Tools.Nunit AssemblyName="MyTestAssembly.dll" 
    ///                    Version="2.2"
    ///                 Fixture=""
    ///                 IncludeCategory="bvt"
    ///                 ExcludeCategory="stress"
    ///                 OutputXmlFileName="MyTestAssemblyResults.xml">
    ///                    <Output TaskParameter="Pass" ItemName="TestPass"></Output>
    ///                    <Output TaskParameter="Fail" ItemName="TestFail"></Output>
    ///                    <Output TaskParameter="NotRun" ItemName="TestNoRun"></Output>
    ///                    <Output TaskParameter="Total" ItemName="TestTotal"></Output>
    ///                    <Output TaskParameter="TimeTaken" ItemName="TestTimeTaken"></Output>
    ///         </Tools.Nunit> 
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>
    public class NUnit : ToolTaskBase
    {
        private const string DefaultCategory = @"{0F0A93AC-97D2-4a70-9993-290E3FC590FC}";

        //private ArrayList testCategoryList = new ArrayList();

        private string[] testCategory;
        private int[] testPass;
        private int[] testFail;
        private int[] testNotRun;
        private double[] testTime;
        private int[] testTotal;
        private bool onlyRunningDefault;

        /// <summary>
        /// Initializes a new instance of the NUnit class.
        /// </summary>
        public NUnit()
        {
        }

        [Required]
        public string[] Assemblies
        {
            get { return (string[])Bag["Assemblies"]; }
            set { Bag["Assemblies"] = value; }
        }

        /// <summary>
        /// Path and file to save the test results
        /// </summary>
        /// <value>Fully qualified path name of the folder to output files to</value>
        public string OutputFolder
        {
            get
            {
                object temp = Bag["OutputFolder"];
                return (temp == null) ? String.Empty : (string)temp;
            }
             
            set { Bag["OutputFolder"] = value; }
        }

        /// <summary>
        /// The path to the xsl file to transform the final results
        /// </summary>
        /// <value></value>
        public string OutputXmlTransformFile
        {
            get
            {
                object temp = Bag["OutputXmlTransformFile"];
                return (temp == null) ? String.Empty : (string)temp;
            }
            set { Bag["OutputXmlTransformFile"] = value; }
        }

        /// <summary>
        /// Version of nUnit used for testing
        /// </summary>
        /// <value>2.0 | 2.1 | 2.2</value>
        [Required]
        public string Version
        {
            get { return (string)Bag["Version"]; }
            set { Bag["Version"] = value; }
        }

        /// <summary>
        /// The fixture to run in the assembly specified
        /// </summary>
        /// <value></value>
        public string Fixture
        {
            get
            {
                object temp = Bag["Fixture"];
                return (temp == null) ? String.Empty : (string)temp;
            }
            set { Bag["Fixture"] = value; }
        }

        /// <summary>
        /// Testing category to include
        /// </summary>
        /// <value>Category Name</value>
        public string IncludeCategory
        {
            get
            {
                object temp = Bag["IncludeCategory"];
                return (temp == null) ? String.Empty : (string)temp;
            }
            set { Bag["IncludeCategory"] = value; }
        }

        /// <summary>
        /// Testing category to exclude
        /// </summary>
        /// <value>Category Name</value>
        public string ExcludeCategory
        {
            get
            {
                object temp = Bag["ExcludeCategory"];
                return (temp == null) ? String.Empty : (string)temp;
            }
            set
            {
                throw new NotImplementedException("Exclude Categories not implemented yet.");

                //Bag["ExcludeCategory"] = value;
            }
        }

        /// <summary>
        /// Test categories performed
        /// </summary>
        /// <value>A list of test categories performed</value>
        [Output]
        public string[] Category
        {
            get { return this.testCategory; }
        }

        /// <summary>
        /// Test passed
        /// </summary>
        /// <value>A list of tests passed</value>
        [Output]
        public int[] Pass
        {
            get { return this.testPass; }
        }

        /// <summary>
        /// Tests failed
        /// </summary>
        /// <value>A list of tests failed</value>
        [Output]
        public int[] Fail
        {
            get { return this.testFail; }
        }

        /// <summary>
        /// Tests not ran
        /// </summary>
        /// <value>A list of tests not ran</value>
        [Output]
        public int[] NotRun
        {
            get { return this.testNotRun; }
        }

        /// <summary>
        /// A list of all tests within the specified assembly
        /// </summary>
        /// <value>A complete list of tests</value>
        [Output]
        public int[] Total
        {
            get { return this.testTotal; }
        }

        /// <summary>
        /// Time taken to test the specified assembly
        /// </summary>
        /// <value>Run time</value>
        [Output]
        public double[] TimeTaken
        {
            get { return this.testTime; }
        }

        /// <summary>
        /// The name of the exe we call.
        /// </summary>
        /// <value>The name of the exe we call.</value>
        protected override string ToolName
        {
            get { return "nunit-console.exe"; }
        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            Assembly nunitAssembly = null;
            Type categoryType = null;

            
            
            /* TODO add this back in for multiple assemblies
            //see if they have specified an nunit project 
            if (this.AssemblyName.EndsWith(".nunit"))
            {
                if (!System.IO.File.Exists(this.AssemblyName))
                {
                    throw new TaskException("General.AssemblyNotExist", this.AssemblyName);
                }
            }
            else
            {
                if (!System.IO.File.Exists(this.AssemblyName))
                {
                    throw new TaskException("General.AssemblyNotExist", this.AssemblyName);
                }
                else
                {
                    if (!Utilities.IsValidAssemblyFile(this.AssemblyName))
                    {
                        throw new TaskException("General.AssemblyNotValid", this.AssemblyName);
                    }
                }
            }

            if (this.OutputXmlTransformFile != null)
            {
                if (!System.IO.File.Exists(this.OutputXmlTransformFile))
                {
                    throw new TaskException("File.NotFound", this.OutputXmlTransformFile);
                }
            }
            */
            //call tooltaskbase.
            //base.InternalExecute();

            this.testCategory = IncludeCategory.Split(';');
            //keep them sorted for a BinarySearch later
            Array.Sort(this.testCategory);

            if (this.testCategory.Length == 0 || (this.testCategory.Length == 1 && this.testCategory[0].Length == 0))
            {
                onlyRunningDefault = true;
                this.testCategory = new string[1] { DefaultCategory.Trim().ToLower() };
            }
            else
            {
                nunitAssembly = Assembly.LoadFrom(Path.Combine(this.ToolPath, "nunit.framework.dll"));
                categoryType = nunitAssembly.GetType("NUnit.Framework.CategoryAttribute");
            }

            this.testFail = new int[this.testCategory.Length];
            this.testNotRun = new int[this.testCategory.Length];
            this.testPass = new int[this.testCategory.Length];
            this.testTime = new double[this.testCategory.Length];
            this.testTotal = new int[this.testCategory.Length];

            foreach (string assembly in this.Assemblies)
            {
                bool found = false;
                    
                if (!onlyRunningDefault)
                {
                    //we are running categories so load the test assembly and check to see if any of the types have a category attribute 
                    //that we will be running
                    
                    Assembly testassembly = Assembly.LoadFrom(assembly);
                    Type[] typeList = testassembly.GetExportedTypes();

                    foreach (Type asstype in typeList)
                    {
                        MethodInfo[] methods = asstype.GetMethods();
                        foreach (MethodInfo m in methods)
                        {
                            if (m.IsDefined(categoryType, false))
                            {
                                object[] cats = m.GetCustomAttributes(categoryType, false);

                                for (int i = 0; i < cats.Length; i++)
                                {
                                    PropertyInfo categoryProperty = cats[i].GetType().GetProperty("Name");
                                    string categoryName = (string)categoryProperty.GetValue(cats[i], null);

                                    int searchResults = Array.BinarySearch(this.testCategory, categoryName);
                                    if (searchResults > -1)
                                    {
                                        found = true;
                                        break;
                                    }
                                }
                            }
                            if (found) { break; }
                        }
                        if (found) { break; }
                    }                    
                }

                if (found || onlyRunningDefault)
                {
                    //found our categories in this assembly so run nunit

                    CommandLineBuilder commandLine = new CommandLineBuilder();

                    FileInfo fi = new FileInfo(assembly);

                    string outputXmlFilename = Path.Combine(this.OutputFolder, Path.GetFileNameWithoutExtension(fi.Name) + "." + this.testCategory[0] + ".TestResult.xml");
                    // supports /fixture /include /exclude /assembly (which is blank) /transform /xml

                    this.GenerateCommandLineCommands(commandLine, assembly, outputXmlFilename);
                    base.ExecuteTool(this.ComputePathToTool(), String.Empty, commandLine.ToString());
                                    
                    if (!System.IO.File.Exists(outputXmlFilename))
                    {
                        //We have no output file something bad happened in Nunit.
                        Log.LogMessageFromResources("Nunit.FatalError");
                        this.testFail[0] = 1;
                        return;
                    }

                    this.ProcessXmlResultsFile(assembly, outputXmlFilename);
                }
            }
        }        

        /// <summary>
        /// Full calculated path to tool including exe name
        /// </summary>
        /// <returns>Defaults to default install location of Nunit</returns>
        protected override string GenerateFullPathToTool()
        {
            string nunitPath = string.Empty;
            nunitPath = Path.Combine(this.GetPathToBinFolder(), this.ToolName);
            return nunitPath;
        }

        /// <summary>
        /// Builds the commandline for this tool
        /// </summary>
        /// <param name="commandLine">The object to append the args to.</param>
        /// <param name="assembly"></param>
        /// <param name="outputXmlFilename"></param>
        /// <returns>True if successfully added args.</returns>
        private bool GenerateCommandLineCommands(CommandLineBuilder commandLine, string assembly, string outputXmlFilename)
        {
            // switch on the version number and build appropriate command line

            string nunitVersion = (string)Bag["Version"];

            switch (nunitVersion)
            {
                case "2.2":
                    {
                        // supports /fixture /include /exclude /assembly (which is blank) /transform /xml
                        commandLine.AppendSwitch(assembly);
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/fixture", "Fixture", "=");
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/include", "IncludeCategory", "=");
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/exclude", "ExcludeCategory", "=");
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/transform", "OutputXmlTransformFile", "=");
                        commandLine.AppendSwitch("/xml=" + outputXmlFilename);

                        break;
                    }
                case "2.1":
                    {
                        commandLine.AppendSwitch(assembly);
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/fixture", "Fixture", "=");
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/transform", "OutputXmlTransformFile", "=");
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/xml", "OutputXmlFilename", "=");
                        break;
                    }
                case "2.0":
                    {
                        commandLine.AppendSwitch("/assembly:" + assembly);
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/fixture", "Fixture", ":");
                        AppendSwitchAndValueWhenNotStringEmpty(commandLine, "/transform", "OutputXmlTransformFile", ":");
                        commandLine.AppendSwitch("/xml:" + outputXmlFilename);
                        break;
                    }
            }
            return true;
        }

        /// <summary>
        /// Processes the nunit results
        /// </summary>
        private void ProcessXmlResultsFile(string assemblyName, string outputXmlFilename)
        {
            XmlDocument doc = new XmlDocument();

            try
            {
                doc.Load(outputXmlFilename);
            }
            catch (Exception e)
            {
                Log.LogErrorFromException(e);
                return;
            }

            XmlNode root = doc.DocumentElement;
            int total = Convert.ToInt32(root.Attributes["total"].Value);
            int failures = Convert.ToInt32(root.Attributes["failures"].Value);

            if (failures != 0)
            {
                Log.LogMessageFromResources("Nunit.UnitTestFailed", assemblyName);
            }

            int notrun = Convert.ToInt32(root.Attributes["not-run"].Value);

            if (this.onlyRunningDefault)
            {
                this.testTotal[0] = total;
                this.testFail[0] = failures;
                this.testNotRun[0] = notrun;
                this.testPass[0] = total - failures;
                this.testTime[0] = Convert.ToDouble(root.ChildNodes[0].Attributes["time"].Value);

                return;
            }

            XmlNode resultsnode = doc.SelectSingleNode(@"test-results");
            XmlNodeList testCaseNodes = doc.SelectNodes(@"//test-case");
            
            foreach (XmlNode node in testCaseNodes)
            {
                bool success = false;
                double timeTaken = 0;

                string testcaseName = node.Attributes["name"].Value;

                
                bool executed = Convert.ToBoolean(node.Attributes["executed"].Value);
                if (executed)
                {
                    timeTaken = Convert.ToDouble(node.Attributes["time"].Value);
                    success = Convert.ToBoolean(node.Attributes["success"].Value);
                }
                
                for (int index = 0; index < this.testCategory.Length; index++)
                {
                    string category = testCategory[index].ToLowerInvariant();

                    foreach (XmlNode bob in node.SelectNodes("categories/category"))
                    {
                        if (bob.Attributes["name"].Value.ToLowerInvariant() == category)
                        {
                            this.testTotal[index] = this.testTotal[index] + 1;
                            if (!executed)
                            {
                                this.testNotRun[index] = this.testNotRun[index] + 1;
                            }
                            else
                            {
                                if (success)
                                {
                                    this.testPass[index] = this.testPass[index] + 1;
                                }
                                else
                                {
                                    this.testFail[index] = this.testFail[index] + 1;
                                }
                                this.testTime[index] = this.testTime[index] + timeTaken;
                            }
                            break;
                        }
                    }
                }
            }
        }        

        /// <summary>
        /// Find the correct path containing the version of nunit specified.
        /// </summary>
        /// <returns>nUnit bin directory</returns>
        private string GetPathToBinFolder()
        {
            string nunitPath = string.Empty;

            switch (this.Version)
            {
                case "2.1":
                    {
                        nunitPath = System.Environment.ExpandEnvironmentVariables(@"%ProgramFiles%\Nunit v2.1\bin");
                        break;
                    }
                case "2.2":
                    {
                        nunitPath = System.Environment.ExpandEnvironmentVariables(@"%ProgramFiles%\Nunit 2.2\bin");
                        break;
                    }
                case "2.0":
                    {
                        nunitPath = System.Environment.ExpandEnvironmentVariables(@"%ProgramFiles%\Nunit v2.0\bin");
                        break;
                    }
                default:
                    {
                        nunitPath = System.Environment.ExpandEnvironmentVariables(@"%ProgramFiles%\Nunit v2.1\bin");
                        break;
                    }
            }
            return nunitPath;
        }
    }
}