﻿//-----------------------------------------------------------------------
// <copyright file="RegisterVirtualMachine.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Mark Phillips</author>
// <email>v-mphill@microsoft.com</email>
// <date>2004-05-24</date>
// <summary>Starts a virtual machine.</summary>
//-----------------------------------------------------------------------

namespace Microsoft.Sdc.Tasks.VirtualServer
{
    #region Using directives
    using System;
    using System.Reflection;
    using System.Collections;
    using System.Globalization;
    using System.Runtime.InteropServices;
    using Microsoft.Sdc.Tasks.VirtualServer;
    using Microsoft.Build.Framework;
    #endregion

    /// <summary>
    /// TODO
    /// Starts a virtual machine.
    /// </summary>          
    /// <remarks>
    /// <code><![CDATA[
    /// <VirtualServer.VirtualMachine.Start
    ///             MachineName="machineName"
    ///             ServerName="serverName"
    ///             Timeout="timeout">
    /// </VirtualServer.VirtualMachine.Start>
    /// ]]></code>
    /// <para>where:</para>
    /// <para><i>machineName (Required)</i></para>
    /// <para>The name of the virtual machine to be started.</para>
    /// <para><i>serverName</i></para>
    /// <para>Set the <i>serverName</i> property to specify a remote server to connect to. Defaults to connect locally.</para>
    /// <para><i>timeout</i></para>
    /// <para>Length of time to wait for the start task to complete, in milliseconds. Defaults to 10.</para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <VirtualServer.VirtualMachine.Start
    ///             MachineName="myMachine"
    ///             Timeout="20"
    ///         </VirtualServer.VirtualMachine.Start> 
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>
    public class RegisterVirtualMachine : VirtualServerBase
    {
        private string serverName;
        private string configurationPath;
        private string configurationName;
        private int timeout = 10; //store in seconds and multiply later

        /// <summary>
        /// Initializes a new instance of the RegisterVirtualMachine class.
        /// </summary>
        public RegisterVirtualMachine()
        {
        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            if (this.ConfigurationPath == null || this.ConfigurationPath.Length == 0)
            {
                //TODO strings
                throw new TaskException("VirtualServer.InvalidMachineName", null);
            }

            //TODO check to see if it exists already and throw if it does

            ComWrapper virtualServer = GetVirtualServer(this.serverName);

            ComWrapper virtualMachine = new ComWrapper(virtualServer.CallMethod("RegisterVirtualMachine", new object[] { String.Empty, this.configurationPath }), true);

            string fullPath = (string)virtualMachine.GetProperty("File");
            if (fullPath.ToLowerInvariant() != configurationPath.ToLowerInvariant())
            {
                throw new TaskException("VirtualServer.TODO");
            }
        }

        /// <summary>
        /// Length of time to wait for the register task to complete, in seconds.
        /// </summary>
        /// <value>Length of time to wait for the start task to complete, in seconds. Defaults to 10.</value>
        public int Timeout
        {
            get
            {
                return this.timeout;
            }
            set
            {
                this.timeout = value;
            }
        }


        /// <summary>
        /// Set the <i>serverName</i> property to specify a remote server to connect to.
        /// </summary>
        /// <value>Set the <i>serverName</i> property to specify a remote server to connect to. Defaults to connect locally.</value>
        public string ServerName
        {
            get
            {
                return (serverName == null ? String.Empty : serverName);
            }
            set
            {
                serverName = value;
            }
        }

        /// <summary>
        /// The fullpath of the virtual machine to be registered with the server (including the .vmc extension).
        /// </summary>
        /// <value>The fullpath of the virtual machine to be registered.</value>
        [Required]
        public string ConfigurationPath
        {
            get
            {
                return (this.configurationPath == null ? String.Empty : this.configurationPath);
            }
            set
            {
                this.configurationPath = value;
            }
        }

        public string ConfigurationName
        {
            get
            {
                return (this.configurationName == null ? String.Empty : this.configurationName);
            }
            set
            {
                this.configurationName = value;
            }
        }

    }
}
