//-----------------------------------------------------------------------
// <copyright file="Start.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Mark Phillips</author>
// <email>v-mphill@microsoft.com</email>
// <date>2004-05-24</date>
// <summary>Starts a virtual machine.</summary>
//-----------------------------------------------------------------------

namespace Microsoft.Sdc.Tasks.VirtualServer.VirtualMachine
{
    #region Using directives
    using System;
    using System.Reflection;
    using System.Collections;
    using System.Globalization;
    using System.Runtime.InteropServices;
    //using Microsoft.VirtualServer.Interop;
    using Microsoft.Sdc.Tasks.VirtualServer;
    using Microsoft.Build.Framework;
    #endregion

    /// <summary>
    /// Starts a virtual machine.
    /// </summary>          
    /// <remarks>
    /// <code><![CDATA[
    /// <VirtualServer.VirtualMachine.Start
    ///             MachineName="machineName"
    ///             ServerName="serverName"
    ///             Timeout="timeout">
    /// </VirtualServer.VirtualMachine.Start>
    /// ]]></code>
    /// <para>where:</para>
    /// <para><i>machineName (Required)</i></para>
    /// <para>The name of the virtual machine to be started.</para>
    /// <para><i>serverName</i></para>
    /// <para>Set the <i>serverName</i> property to specify a remote server to connect to. Defaults to connect locally.</para>
    /// <para><i>timeout</i></para>
    /// <para>Length of time to wait for the start task to complete, in milliseconds. Defaults to 10.</para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <VirtualServer.VirtualMachine.Start
    ///             MachineName="myMachine"
    ///             Timeout="20">
    ///         </VirtualServer.VirtualMachine.Start> 
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>
    public class Start : VirtualServerBase
    {
        private string serverName;
        private string machineName;
        private int timeout = 10;

        /// <summary>
        /// Initializes a new instance of the VirtualServer.Start class.
        /// </summary>
        public Start()
        {

        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            if (this.machineName == null || this.machineName.Length == 0)
            {
                throw new TaskException("VirtualServer.InvalidMachineName", null);
            }
            
            ComWrapper virtualMachines = GetVirtualMachinesFromServer(this.serverName);
            ComWrapper virtualMachine = GetVirtualMachine(virtualMachines, this.machineName);

            NativeMethods.VMState state = (NativeMethods.VMState)virtualMachine.GetProperty("State");

            if (state == NativeMethods.VMState.VMState_TurnedOff)
            {
                ComWrapper task = new ComWrapper(virtualMachine.CallMethod("Startup"), true);
                task.CallMethod("WaitForCompletion", new object[] { timeout * 1000 });

                state = (NativeMethods.VMState)virtualMachine.GetProperty("State");

                if (state == NativeMethods.VMState.VMState_Running)
                {
                    Log.LogMessage(MessageImportance.Normal, String.Format("Machine {0} has been started.", this.MachineName));
                }
                else
                {
                    throw new TaskException("VirtualServer.StartFailure", MachineName);
                }
            }
            else
            {
                Log.LogMessage(MessageImportance.Normal, String.Format(CultureInfo.InvariantCulture, "Machine {0} is already running.", MachineName));
            }
        }

        /// <summary>
        /// Length of time to wait for the start task to complete, in seconds.
        /// </summary>
        /// <value>Length of time to wait for the start task to complete, in seconds. Defaults to 10.</value>
        public int Timeout
        {
            get
            {
                return timeout;
            }
            set
            {
                timeout = value;
            }
        }


        /// <summary>
        /// Set the <i>serverName</i> property to specify a remote server to connect to.
        /// </summary>
        /// <value>Set the <i>serverName</i> property to specify a remote server to connect to. Defaults to connect locally.</value>
        public string ServerName
        {
            get
            {
                return (serverName == null ? String.Empty : serverName);
            }
            set
            {
                serverName = value;
            }
        }

        /// <summary>
        /// The name of the virtual machine to be started.
        /// </summary>
        /// <value>The name of the virtual machine to be started.</value>
        [Required]
        public string MachineName
        {
            get
            {
                return (machineName == null ? String.Empty : machineName);
            }
            set
            {
                machineName = value;
            }
        }

    }
}