//-----------------------------------------------------------------------
// <copyright file="Exists.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Andy Reeves</author>
// <email>andyr</email>
// <date>2004-03-23</date>
// <summary>Checks whether an Applcation Pool exists (IIS6 or later).</summary>
//-----------------------------------------------------------------------
namespace Microsoft.Sdc.Tasks.Web.AppPool
{
    using System;
    using Microsoft.Build.Framework;
    using Web = Microsoft.Sdc.Tasks.Configuration.Web;

    /// <summary>
    /// Checks whether an Applcation Pool exists (IIS6 or later).
    /// </summary>
    /// <remarks>
    /// <code><![CDATA[<Web.AppPool.Delete AppPoolName="appPoolName"/>]]></code>
    /// <para>where:</para>
    /// <para><i>appPoolName (Required)</i></para>
    /// <para>A valid, existing IIS pool name</para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <Web.AppPool.Exists
    ///             AppPoolName="TestApplicationPool">
    ///             <Output TaskParameter="DoesExist" PropertyName="AppPoolExists"/>
    ///         </Web.AppPool.Exists>
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>    
    public class Exists : TaskBase
    {
        private string name;
        private bool doesExist;
        private string machineName = "localhost";

        /// <summary>
        /// Machine to operate on
        /// </summary>
        public string MachineName
        {
            get { return this.machineName; }
            set { this.machineName = value; }
        }

        /// <summary>
        /// Name of the application pool to check for.
        /// </summary>
        /// <value>Any valid IIS application pool name</value>
        [Required]
        public string AppPoolName
        {
            get { return this.name ?? String.Empty; }

            set { this.name = value; }
        }

        /// <summary>
        /// Gets whether the app pool does exist.
        /// </summary>
        /// <value>true if the app pool exists, or false otherwise.</value>
        [Output]
        public bool DoesExist
        {
            get { return this.doesExist; }
        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            Log.LogMessage(MessageImportance.Normal, string.Format("Checking whether AppPool: {0} exists on: {1}", this.AppPoolName, this.MachineName));
            this.doesExist = Web.AppPool.Exists(this.name, this.MachineName);
        }
    }
}