//-----------------------------------------------------------------------
// <copyright file="Start.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Mark Phillips</author>
// <email>andyr</email>
// <date>2004-03-23</date>
// <summary>Starts a stopped FtpSite.</summary>
//-----------------------------------------------------------------------

namespace Microsoft.Sdc.Tasks.Web.FtpSite
{
    using System;
    using System.Xml;
    using System.Globalization;
    using System.Net;
    using Microsoft.Sdc.Tasks.Configuration.Web;
    using Microsoft.Build.Framework;

    /// <summary>
    /// Starts a stopped ftp site
    /// </summary>
    /// <remarks>
    /// <code><![CDATA[<Web.FtpSite.Start 
    ///     Description="description" 
    ///     Identifier="identifier")>]]></code>
    /// <para>where:</para>
    /// <para><i>description</i></para>
    /// <para>A description for the web site.</para>
    /// <para><i>identifier</i></para>
    /// <para>A valid ftp site identifier.</para>
    /// <para><b>Either a <i>description</i> or an <i>identifier</i> must be specified.</b></para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <Web.FtpSite.Start Description="ftptestsite" />
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>    
    public class Start : TaskBase
    {
        private string description;
        private int id;
        private string identifier;
        private Microsoft.Sdc.Tasks.Configuration.Web.FtpSite ftp;

        /// <summary>
        /// Initializes a new instance of the StartFtpSite class.
        /// </summary>
        public Start()
        {
        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            #region Execute code
            if (description == null && identifier == null)
            {
                throw new TaskException("Web.DescriptionOrIdentifier");
            }
            if (this.description != null && this.identifier != null)
            {
                throw new TaskException("Web.DescriptionOrIdentifierNotBoth", description, identifier);
            }
            id = Utilities.CheckFtpSiteDescriptionOrIdentifier(description, identifier, "Starting");
            if (Microsoft.Sdc.Tasks.Configuration.Web.FtpSite.Exists(id))
            {
                ftp = Microsoft.Sdc.Tasks.Configuration.Web.FtpSite.Load(id);
                ftp.Start();
            }
            #endregion
        }

        /// <summary>
        /// Description of ftp site to start
        /// </summary>
        /// <value>A valid ftp site description</value>
        /// <remarks>If this parameter is not specified you must specify the site <seealso cref="Identifier">Identifier</seealso> 
        /// for the site you wish to start</remarks>
        public string Description
        {
            get
            {
                return (description == null ? String.Empty : description);
            }
            set
            {
                description = value;
            }
        }

        /// <summary>
        /// ftp site idenfitier to start
        /// </summary>
        /// <value>A valid ftp site identifier</value>
        /// <remarks>If this parameter is not specified you must specify the site <seealso cref="Description">Description</seealso> 
        /// for the site you wish to start</remarks>
        public string Identifier
        {
            get
            {
                return (identifier == null ? String.Empty : identifier);
            }
            set
            {
                identifier = value;
            }
        }
    }
}