//-----------------------------------------------------------------------
// <copyright file="UnloadVirtualDirectory.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <summary></summary>
//-----------------------------------------------------------------------

namespace Microsoft.Sdc.Tasks.Web.WebSite
{
    using System;
    using Microsoft.Sdc.Tasks.Configuration.Web;
    using Microsoft.Build.Framework;

    /// <summary>
    /// Unloads the IIS application from the specified virtual driectory
    /// </summary>
    /// <remarks>
    /// <code><![CDATA[<Web.WebSite.UnloadVirtualDirectory
    ///   VirtualDirectoryName="virtualDirectoryName"
    ///   MachineName="machineName"
    ///   WebSiteName="webSiteName" /> ]]></code>
    /// <para>where:</para>
    /// <para><i>virtualDirectoryName (required)</i></para>
    /// <para>The virtual directory to remove the application from</para>
    /// <para><i>machineName</i></para>
    /// <para>Machine to perform the action on</para>
    /// <para><i>websiteName</i></para>
    /// <para>Web site to perform the action on</para>
    /// </remarks>
    /// <example>
    /// <code><![CDATA[
    /// <Project>
    ///     <Target Name="Test" >
    ///         <Web.WebSite.UnloadVirtualDirectory
    ///             VirtualDirectoryName="/test"
    ///             WebSiteName="testwebsite" />
    ///     </Target>
    /// </Project>
    /// ]]></code>    
    /// </example>    
    public class UnloadVirtualDirectory : TaskBase
    {
        private string machineName = "localhost";
        private string webSiteName = "Default Web Site";
        private string name = String.Empty;

        /// <summary>
        /// Virtual directory to unload the application from
        /// </summary>
        /// <value>A valid, existing virtual directory</value>
        [Required]
        public string VirtualDirectoryName
        {
            get { return (this.name ?? String.Empty); }
            set { this.name = value; }
        }

        /// <summary>
        /// Machine to perform the action on
        /// </summary>
        /// <value>A valid machine name. If this value is not set the delete will 
        /// target on the local machine.</value>
        public string MachineName
        {
            get { return (this.machineName ?? String.Empty); }
            set { this.machineName = value; }
        }

        /// <summary>
        /// Web site to unload the virtual directory's application from
        /// </summary>
        /// <value>A value web site name. If this value is not set the delete will
        /// target against "Default Web Site"</value>
        public string WebSiteName
        {
            get { return (this.webSiteName ?? String.Empty); }
            set { this.webSiteName = value; }
        }

        /// <summary>
        /// Performs the action of this task.
        /// </summary>
        protected override void InternalExecute()
        {
            WebSite web = WebSite.Load(this.machineName, this.webSiteName);
            VirtualDirectory vdir = web.VirtualDirectories[this.name];
            if (vdir != null)
            {
                Log.LogMessageFromResources("Web.AppUnloadVitualDirectory", this.name);
                vdir.AppUnLoad();
            }
        }
    }
}