﻿//-----------------------------------------------------------------------
// <copyright file="DeleteAttribute.cs" company="Microsoft">
// Copyright (c) Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY 
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
// </copyright>
// <author>Steve Overall</author>
// <email>v-stevov</email>
// <date>2005-04-15</date>
// <summary>Deletes an existing attribute from an XML file.</summary>
//-----------------------------------------------------------------------

namespace Microsoft.Sdc.Tasks.XmlFile
{
	#region Using directives
	using System;
	using System.Collections.Generic;
	using System.Text;
	using System.Xml;

	using Microsoft.Build.Framework;
	using Microsoft.Build.Tasks;
	using Microsoft.Build.Utilities;
	using Microsoft.Sdc.Tasks;

	#endregion

	#region class comments
	/// <summary>
	/// Deletes the named attribute from all elements in the specified file
	/// that match the specified XPath expression.
	/// Will fail of no matches found for the XPath expression.
	/// </summary>
	/// <remarks>
	/// <code>
	/// <![CDATA[<XmlFile.DeleteAttribute Path="path" XPath="xpath" Name="name" Namespaces="namespaces" IgnoreNoMatchFailure="ignoreNoMatchFailure" Force="force"/>]]>
	/// </code>
	/// <para>where:</para>
	/// <para><i>Path (Required)</i></para>
	/// <para>The path of the XML file to modify.</para>
	/// <para><i>XPath (Required)</i></para>
	/// <para>The XPath expression to evaluate to find the parent element(s) to remove the attribute from.</para>
	/// <para><i>Name (Required)</i></para>
	/// <para>The name of the attribute to delete.</para>
	/// <para><i>Namespaces</i></para>
	/// <para>
	/// Optional TaskItems array detailing each of the namespace declarations required to read/maintain the target document.
	/// Each TaskItem should have 2 elements &lt;Prefix&gt; and &lt;Uri&gt;.
	/// </para>
	/// <para><i>IgnoreNoMatchFailure</i></para>
	/// <para>If set to true, no Exception will be thrown if no matches made for the supplied XPath.  Defaults to false.</para>
	/// <para><i>Force</i></para>
	/// <para>If set to true, this will force the file to be updated even if it is Read Only.  Defaults to false.</para>
	/// </remarks>
	/// <example>
	/// <code><![CDATA[
	/// <Project>
	///     <Target Name="Test" >
	///         <XmlFile.DeleteAttribute 
	///				Path="c:\app.config"
	///				XPath="/configuration/appSettings/add"
	///				Name="key"
	///				Namespaces="@(ConfigNamespaces)"
	///				IgnoreNoMatchFailure="true"
	///				Force="true"
	///			/>
	///     </Target>
	/// </Project>
	/// ]]></code>    
	/// </example>
	#endregion
	public class DeleteAttribute : BaseXmlFileTask
	{
		#region private declarations
		private string		_name					= String.Empty;
		private bool		_ignoreNoMatchFailure	= false;
		private bool		_force					= false;
		#endregion

		#region constructors
		/// <summary>
		/// Create a new DeleteAttribute.
		/// </summary>
		public DeleteAttribute() : base()
		{
		}
		#endregion

		#region override of InternalExecute
		/// <summary>
		/// Carry out this task's action.
		/// </summary>
		protected override void InternalExecute()
		{
			//	Load the xml file.
			XmlDocument xmlDoc = this.OpenXmlFile( this.Path );
			XmlNamespaceManager nsMgr = this.GetNamespaceManagerForDoc( xmlDoc, this.Namespaces );

			//	Run the XPath Expression against the xml file.
			//	Get the list of matching elements.
			XmlNodeList elements = this.SelectXmlNodesOfType( xmlDoc, this.XPath, XmlNodeType.Element, nsMgr );
			if (elements != null && elements.Count > 0)
			{
				//	Remove each element that matches the XPath Expression.
				int attributeIx = 0;
				foreach ( XmlNode element in elements )
				{
					XmlNode attribute = element.Attributes[_name];
					if ( attribute != null )
					{
						element.Attributes.Remove( element.Attributes[_name] );
					}
					else
					{
						//	Attribute not found.  Issue a warning.
						Log.LogWarningFromResources( "XmlFile.DeleteAttribute.NotFound.Warning", this.FileName, _name, attributeIx, this.XPath );
					}

					++attributeIx;
				}

				//	Save the Xml Doc back to disc.
				this.SaveXmlFile( xmlDoc, this.Path, _force );

				Log.LogMessageFromResources( "XmlFile.DeleteAttribute.Success", this.FileName, _name, elements.Count, this.XPath );
			}
			else
			{
				if ( _ignoreNoMatchFailure )
					Log.LogWarningFromResources( "XmlFile.NoMatch.Warning", this.FileName, this.XPath );
				else
					throw new TaskException( Log.FormatResourceString( "XmlFile.NoMatch.Exception", this.XPath ) );
			}
		}
		#endregion

		#region public Properties
		/// <summary>
		/// The name of the new element to append to each element matching the XPath Expression.
		/// </summary>
		/// <value>The name of the new element to append to each element matching the XPath Expression.</value>
		[Required]
		public string Name
		{
			get { return _name; }
			set { _name = (value != null ? value : String.Empty); }
		}

		/// <summary>
		/// If set to true, no Exception will be thrown if no matches made for the supplied XPath.  Defaults to false.
		/// </summary>
		/// <value>If set to true, no Exception will be thrown if no matches made for the supplied XPath.  Defaults to false.</value>
		public bool IgnoreNoMatchFailure
		{
			get { return _ignoreNoMatchFailure; }
			set { _ignoreNoMatchFailure = value; }
		}

		/// <summary>
		/// If set to true, this will modify the xml file even if it is read-only.  Defaults to false.
		/// </summary>
		/// <value>If set to true, this will modify the xml file even if it is read-only.  Defaults to false.</value>
		public bool Force
		{
			get { return _force; }
			set { _force = value; }
		}
		#endregion
	}
}
